@@ -122,6 +122,7 @@ from doit.cmd_base import ModuleTaskLoader
 from doit.reporter import ZeroReporter
 from doit.exceptions import TaskError
 from doit.api import run_tasks
+from doit import task_params
 from pydevtool.cli import UnifiedContext, CliGroup, Task
 from rich.console import Console
 from rich.panel import Panel
@@ -188,7 +189,7 @@ rich_click.COMMAND_GROUPS = {
         },
         {
             "name": "documentation",
-            "commands": ["doc", "refguide-check"],
+            "commands": ["doc", "refguide-check", "smoke-docs", "smoke-tutorial"],
         },
         {
             "name": "release",
@@ -430,6 +431,15 @@ class Build(Task):
         ['--with-scipy-openblas'], default=False, is_flag=True,
         help=("If set, use the `scipy-openblas32` wheel installed into the "
               "current environment as the BLAS/LAPACK to build against."))
+    with_accelerate = Option(
+        ['--with-accelerate'], default=False, is_flag=True,
+        help=("If set, use `Accelerate` as the BLAS/LAPACK to build against."
+              " Takes precedence over -with-scipy-openblas (macOS only)")
+    )
+    tags = Option(
+        ['--tags'], default="runtime,python-runtime,tests,devel",
+        show_default=True, help="Install tags to be used by meson."
+    )
 
     @classmethod
     def setup_build(cls, dirs, args):
@@ -482,10 +492,13 @@ class Build(Task):
             cmd += ['-Db_sanitize=address,undefined']
         if args.setup_args:
             cmd += [str(arg) for arg in args.setup_args]
-        if args.with_scipy_openblas:
+        if args.with_accelerate:
+            # on a mac you probably want to use accelerate over scipy_openblas
+            cmd += ["-Dblas=accelerate"]
+        elif args.with_scipy_openblas:
             cls.configure_scipy_openblas()
             env['PKG_CONFIG_PATH'] = os.pathsep.join([
-                    os.path.join(os.getcwd(), '.openblas'),
+                    os.getcwd(),
                     env.get('PKG_CONFIG_PATH', '')
                     ])
 
@@ -535,7 +548,8 @@ class Build(Task):
             if non_empty and not dirs.site.exists():
                 raise RuntimeError("Can't install in non-empty directory: "
                                    f"'{dirs.installed}'")
-        cmd = ["meson", "install", "-C", args.build_dir, "--only-changed"]
+        cmd = ["meson", "install", "-C", args.build_dir,
+               "--only-changed", "--tags", args.tags]
         log_filename = dirs.root / 'meson-install.log'
         start_time = datetime.datetime.now()
         cmd_str = ' '.join([str(p) for p in cmd])
@@ -593,13 +607,12 @@ class Build(Task):
 
     @classmethod
     def configure_scipy_openblas(self, blas_variant='32'):
-        """Create .openblas/scipy-openblas.pc and scipy/_distributor_init_local.py
+        """Create scipy-openblas.pc and scipy/_distributor_init_local.py
 
         Requires a pre-installed scipy-openblas32 wheel from PyPI.
         """
         basedir = os.getcwd()
-        openblas_dir = os.path.join(basedir, ".openblas")
-        pkg_config_fname = os.path.join(openblas_dir, "scipy-openblas.pc")
+        pkg_config_fname = os.path.join(basedir, "scipy-openblas.pc")
 
         if os.path.exists(pkg_config_fname):
             return None
@@ -617,9 +630,8 @@ class Build(Task):
         with open(local, "w", encoding="utf8") as fid:
             fid.write(f"import {module_name}\n")
 
-        os.makedirs(openblas_dir, exist_ok=True)
         with open(pkg_config_fname, "w", encoding="utf8") as fid:
-            fid.write(openblas.get_pkg_config().replace("\\", "/"))
+            fid.write(openblas.get_pkg_config())
 
     @classmethod
     def run(cls, add_path=False, **kwargs):
@@ -689,7 +701,8 @@ class Test(Task):
         ['--array-api-backend', '-b'], default=None, metavar='ARRAY_BACKEND',
         multiple=True,
         help=(
-            "Array API backend ('all', 'numpy', 'pytorch', 'cupy', 'array_api_strict')."
+            "Array API backend "
+            "('all', 'numpy', 'pytorch', 'cupy', 'array_api_strict', 'jax.numpy')."
         )
     )
     # Argument can't have `help=`; used to consume all of `-- arg1 arg2 arg3`
@@ -738,8 +751,8 @@ class Test(Task):
         runner, version, mod_path = get_test_runner(PROJECT_MODULE)
         # FIXME: changing CWD is not a good practice
         with working_dir(dirs.site):
-            print("Running tests for {} version:{}, installed at:{}".format(
-                        PROJECT_MODULE, version, mod_path))
+            print(f"Running tests for {PROJECT_MODULE} version:{version}, "
+                  f"installed at:{mod_path}")
             # runner verbosity - convert bool to int
             verbose = int(args.verbose) + 1
             result = runner(  # scipy._lib._testutils:PytestTester
@@ -761,6 +774,133 @@ class Test(Task):
         return cls.scipy_tests(args, pytest_args)
 
 
+@cli.cls_cmd('smoke-docs')
+class SmokeDocs(Task):
+    # XXX This essntially is a copy-paste of the Task class. Consider de-duplicating.
+    ctx = CONTEXT
+
+    verbose = Option(
+        ['--verbose', '-v'], default=False, is_flag=True,
+        help="more verbosity")
+    durations = Option(
+        ['--durations', '-d'], default=None, metavar="NUM_TESTS",
+        help="Show timing for the given number of slowest tests"
+    )
+    submodule = Option(
+        ['--submodule', '-s'], default=None, metavar='MODULE_NAME',
+        help="Submodule whose tests to run (cluster, constants, ...)")
+    tests = Option(
+        ['--tests', '-t'], default=None, multiple=True, metavar='TESTS',
+        help='Specify tests to run')
+    parallel = Option(
+        ['--parallel', '-j'], default=1, metavar='N_JOBS',
+        help="Number of parallel jobs for testing"
+    )
+    # Argument can't have `help=`; used to consume all of `-- arg1 arg2 arg3`
+    pytest_args = Argument(
+        ['pytest_args'], nargs=-1, metavar='PYTEST-ARGS', required=False
+    )
+
+    TASK_META = {
+        'task_dep': ['build'],
+    }
+
+    @classmethod
+    def scipy_tests(cls, args, pytest_args):
+        dirs = Dirs(args)
+        dirs.add_sys_path()
+        print(f"SciPy from development installed path at: {dirs.site}")
+
+        # FIXME: support pos-args with doit
+        extra_argv = list(pytest_args[:]) if pytest_args else []
+        if extra_argv and extra_argv[0] == '--':
+            extra_argv = extra_argv[1:]
+
+        if args.durations:
+            extra_argv += ['--durations', args.durations]
+
+        # convert options to test selection
+        if args.submodule:
+            tests = [PROJECT_MODULE + "." + args.submodule]
+        elif args.tests:
+            tests = args.tests
+        else:
+            tests = None
+
+        # use strategy=api unless -t path/to/specific/file
+        if not args.tests:
+            extra_argv += ['--doctest-collect=api']
+
+        runner, version, mod_path = get_test_runner(PROJECT_MODULE)
+        # FIXME: changing CWD is not a good practice
+        with working_dir(dirs.site):
+            print(f"Running tests for {PROJECT_MODULE} version:{version}, "
+                  f"installed at:{mod_path}")
+            # runner verbosity - convert bool to int
+            verbose = int(args.verbose) + 1
+            result = runner(  # scipy._lib._testutils:PytestTester
+                "fast",
+                verbose=verbose,
+                extra_argv=extra_argv,
+                doctests=True,
+                coverage=False,
+                tests=tests,
+                parallel=args.parallel)
+        return result
+
+    @classmethod
+    def run(cls, pytest_args, **kwargs):
+        """run unit-tests"""
+        kwargs.update(cls.ctx.get())
+        Args = namedtuple('Args', [k for k in kwargs.keys()])
+        args = Args(**kwargs)
+        return cls.scipy_tests(args, pytest_args)
+
+
+@cli.cls_cmd('smoke-tutorials')
+class SmokeTutorials(Task):
+    """:wrench: Run smoke-tests on tutorial files."""
+    ctx = CONTEXT
+
+    tests = Option(
+        ['--tests', '-t'], default=None, multiple=True, metavar='TESTS',
+        help='Specify *rst files to smoke test')
+    verbose = Option(
+        ['--verbose', '-v'], default=False, is_flag=True, help="verbosity")
+
+    pytest_args = Argument(
+        ['pytest_args'], nargs=-1, metavar='PYTEST-ARGS', required=False
+    )
+
+    @classmethod
+    def task_meta(cls, **kwargs):
+        kwargs.update(cls.ctx.get())
+        Args = namedtuple('Args', [k for k in kwargs.keys()])
+        args = Args(**kwargs)
+        dirs = Dirs(args)
+
+        cmd = ['pytest']
+        if args.tests:
+            cmd += list(args.tests)
+        else:
+            cmd += ['doc/source/tutorial', '--doctest-glob=*rst']
+        if args.verbose:
+            cmd += ['-v']
+
+        pytest_args = kwargs.pop('pytest_args', None)
+        extra_argv = list(pytest_args[:]) if pytest_args else []
+        if extra_argv and extra_argv[0] == '--':
+            extra_argv = extra_argv[1:]
+        cmd += extra_argv
+
+        cmd_str = ' '.join(cmd)
+        return {
+            'actions': [f'env PYTHONPATH={dirs.site} {cmd_str}'],
+            'task_dep': ['build'],
+            'io': {'capture': False},
+        }
+
+
 @cli.cls_cmd('bench')
 class Bench(Task):
     """:wrench: Run benchmarks.
@@ -906,17 +1046,37 @@ def emit_cmdstr(cmd):
     console.print(f"{EMOJI.cmd} [cmd] {cmd}")
 
 
-def task_lint():
+@task_params([{"name": "fix", "default": False}])
+def task_lint(fix):
     # Lint just the diff since branching off of main using a
     # stricter configuration.
     # emit_cmdstr(os.path.join('tools', 'lint.py') + ' --diff-against main')
+    cmd = str(Dirs().root / 'tools' / 'lint.py') + ' --diff-against=main'
+    if fix:
+        cmd += ' --fix'
     return {
         'basename': 'lint',
-        'actions': [str(Dirs().root / 'tools' / 'lint.py') +
-                    ' --diff-against=main'],
+        'actions': [cmd],
         'doc': 'Lint only files modified since last commit (stricter rules)',
     }
 
+@task_params([])
+def task_check_python_h_first():
+    # Lint just the diff since branching off of main using a
+    # stricter configuration.
+    # emit_cmdstr(os.path.join('tools', 'lint.py') + ' --diff-against main')
+    cmd = "{!s} --diff-against=main".format(
+        Dirs().root / 'tools' / 'check_python_h_first.py'
+    )
+    return {
+        'basename': 'check_python_h_first',
+        'actions': [cmd],
+        'doc': (
+            'Check Python.h order only files modified since last commit '
+            '(stricter rules)'
+        ),
+    }
+
 
 def task_unicode_check():
     # emit_cmdstr(os.path.join('tools', 'unicode-check.py'))
@@ -941,11 +1101,17 @@ def task_check_test_name():
 class Lint:
     """:dash: Run linter on modified files and check for
     disallowed Unicode characters and possibly-invalid test names."""
-    def run():
+    fix = Option(
+        ['--fix'], default=False, is_flag=True, help='Attempt to auto-fix errors'
+    )
+
+    @classmethod
+    def run(cls, fix):
         run_doit_task({
-            'lint': {},
+            'lint': {'fix': fix},
             'unicode-check': {},
             'check-testname': {},
+            'check_python_h_first': {},
         })
 
 
@@ -1081,8 +1247,7 @@ class RefguideCheck(Task):
         dirs = Dirs(args)
 
         cmd = [f'{sys.executable}',
-               str(dirs.root / 'tools' / 'refguide_check.py'),
-               '--doctests']
+               str(dirs.root / 'tools' / 'refguide_check.py')]
         if args.verbose:
             cmd += ['-vvv']
         if args.submodule:
