@@ -190,6 +190,10 @@ warnings.filterwarnings(
     message=r'.*path is deprecated.*',
     category=DeprecationWarning,
 )
+# See https://github.com/sphinx-doc/sphinx/issues/12589
+suppress_warnings = [
+    'autosummary.import_cycle',
+]
 
 # -----------------------------------------------------------------------------
 # HTML output
@@ -201,31 +205,35 @@ html_logo = '_static/logo.svg'
 html_favicon = '_static/favicon.ico'
 
 html_sidebars = {
-    "index": "search-button-field",
+    "index": ["search-button-field"],
     "**": ["search-button-field", "sidebar-nav-bs"]
 }
 
 html_theme_options = {
-  "github_url": "https://github.com/scipy/scipy",
-  "twitter_url": "https://twitter.com/SciPy_team",
-  "header_links_before_dropdown": 6,
-  "icon_links": [],
-  "navbar_start": ["navbar-logo", "version-switcher"],
-  "navbar_persistent": [],
-  "switcher": {
-      "json_url": "https://scipy.github.io/devdocs/_static/version_switcher.json",
-      "version_match": version,
-  },
-  "show_version_warning_banner": True,
-  "secondary_sidebar_items": ["page-toc"],
-  # The service https://plausible.io is used to gather simple
-  # and privacy-friendly analytics for the site. The dashboard can be accessed
-  # at https://analytics.scientific-python.org/docs.scipy.org
-  # The Scientific-Python community is hosting and managing the account.
-  "analytics": {
-      "plausible_analytics_domain": "docs.scipy.org",
-      "plausible_analytics_url": "https://views.scientific-python.org/js/script.js",
-  }
+    "github_url": "https://github.com/scipy/scipy",
+    "twitter_url": "https://twitter.com/SciPy_team",
+    "header_links_before_dropdown": 6,
+    "icon_links": [],
+    "logo": {
+        "text": "SciPy",
+    },
+    "navbar_start": ["navbar-logo"],
+    "navbar_end": ["version-switcher", "theme-switcher", "navbar-icon-links"],
+    "navbar_persistent": [],
+    "switcher": {
+        "json_url": "https://scipy.github.io/devdocs/_static/version_switcher.json",
+        "version_match": version,
+    },
+    "show_version_warning_banner": True,
+    "secondary_sidebar_items": ["page-toc"],
+    # The service https://plausible.io is used to gather simple
+    # and privacy-friendly analytics for the site. The dashboard can be accessed
+    # at https://analytics.scientific-python.org/docs.scipy.org
+    # The Scientific-Python community is hosting and managing the account.
+    "analytics": {
+        "plausible_analytics_domain": "docs.scipy.org",
+        "plausible_analytics_url": "https://views.scientific-python.org/js/script.js",
+    },
 }
 
 if 'dev' in version:
@@ -346,7 +354,7 @@ import warnings
 for key in (
         'interp2d` is deprecated',  # Deprecation of scipy.interpolate.interp2d
         'scipy.misc',  # scipy.misc deprecated in v1.10.0; use scipy.datasets
-        'kurtosistest only valid',  # intentionally "bad" excample in docstring
+        '`kurtosistest` p-value may be',  # intentionally "bad" example in docstring
         'scipy.signal.daub is deprecated',
         'scipy.signal.qmf is deprecated',
         'scipy.signal.cascade is deprecated',
