@@ -12,12 +12,25 @@ import pytest
 from pytest import raises as assert_raises
 
 import scipy.sparse
-from scipy.io import mminfo, mmread, mmwrite
+import scipy.io._mmio
+import scipy.io._fast_matrix_market as fmm
+
 
 parametrize_args = [('integer', 'int'),
                     ('unsigned-integer', 'uint')]
 
 
+# Run the entire test suite on both _mmio and _fast_matrix_market implementations
+@pytest.fixture(scope='module', params=(scipy.io._mmio, fmm), autouse=True)
+def implementations(request):
+    global mminfo
+    global mmread
+    global mmwrite
+    mminfo = request.param.mminfo
+    mmread = request.param.mmread
+    mmwrite = request.param.mmwrite
+
+
 class TestMMIOArray:
     def setup_method(self):
         self.tmpdir = mkdtemp()
@@ -50,7 +63,7 @@ class TestMMIOArray:
 
     def test_64bit_integer(self):
         a = array([[2**31, 2**32], [2**63-2, 2**63-1]], dtype=np.int64)
-        if (np.intp(0).itemsize < 8):
+        if (np.intp(0).itemsize < 8) and mmwrite == scipy.io._mmio.mmwrite:
             assert_raises(OverflowError, mmwrite, self.fn, a)
         else:
             self.check_exact(a, (2, 2, 4, 'array', 'integer', 'general'))
@@ -114,6 +127,7 @@ class TestMMIOArray:
         a = np.random.random(sz)
         self.check(a, (20, 15, 300, 'array', 'real', 'general'))
 
+    @pytest.mark.fail_slow(5)
     def test_bad_number_of_array_header_fields(self):
         s = """\
             %%MatrixMarket matrix array real general
@@ -176,7 +190,7 @@ class TestMMIOSparseCSR(TestMMIOArray):
         a = scipy.sparse.csr_matrix(array([[2**32+1, 2**32+1],
                                            [-2**63+2, 2**63-2]],
                                           dtype=np.int64))
-        if (np.intp(0).itemsize < 8):
+        if (np.intp(0).itemsize < 8) and mmwrite == scipy.io._mmio.mmwrite:
             assert_raises(OverflowError, mmwrite, self.fn, a)
         else:
             self.check_exact(a, (2, 2, 4, 'coordinate', 'integer', 'general'))
@@ -347,7 +361,8 @@ class TestMMIOReadLargeIntegers:
         with open(self.fn, 'w') as f:
             f.write(example)
         assert_equal(mminfo(self.fn), info)
-        if (over32 and (np.intp(0).itemsize < 8)) or over64:
+        if ((over32 and (np.intp(0).itemsize < 8) and mmwrite == scipy.io._mmio.mmwrite)
+            or over64):
             assert_raises(OverflowError, mmread, self.fn)
         else:
             b = mmread(self.fn)
@@ -769,3 +784,20 @@ def test_gh18123(tmp_path):
     with open(test_file, "w") as f:
         f.writelines(lines)
     mmread(test_file)
+
+
+def test_threadpoolctl():
+    try:
+        import threadpoolctl
+        if not hasattr(threadpoolctl, "register"):
+            pytest.skip("threadpoolctl too old")
+            return
+    except ImportError:
+        pytest.skip("no threadpoolctl")
+        return
+
+    with threadpoolctl.threadpool_limits(limits=4):
+        assert_equal(fmm.PARALLELISM, 4)
+
+    with threadpoolctl.threadpool_limits(limits=2, user_api='scipy'):
+        assert_equal(fmm.PARALLELISM, 2)
