@@ -219,13 +219,13 @@ def _apply_pivot(T, basis, pivrow, pivcol, tol=1e-9):
     # The selected pivot should never lead to a pivot value less than the tol.
     if np.isclose(pivval, tol, atol=0, rtol=1e4):
         message = (
-            "The pivot operation produces a pivot value of:{: .1e}, "
+            f"The pivot operation produces a pivot value of:{pivval: .1e}, "
             "which is only slightly greater than the specified "
-            "tolerance{: .1e}. This may lead to issues regarding the "
+            f"tolerance{tol: .1e}. This may lead to issues regarding the "
             "numerical stability of the simplex method. "
             "Removing redundant constraints, changing the pivot strategy "
             "via Bland's rule or increasing the tolerance may "
-            "help reduce the issue.".format(pivval, tol))
+            "help reduce the issue.")
         warn(message, OptimizeWarning, stacklevel=5)
 
 
