@@ -5,6 +5,7 @@ __docformat__ = "restructuredtext en"
 __all__ = ['dok_array', 'dok_matrix', 'isspmatrix_dok']
 
 import itertools
+from warnings import warn
 import numpy as np
 
 from ._matrix import spmatrix
@@ -14,28 +15,28 @@ from ._sputils import (isdense, getdtype, isshape, isintlike, isscalarlike,
                        upcast, upcast_scalar, check_shape)
 
 
-class _dok_base(_spbase, IndexMixin):
+class _dok_base(_spbase, IndexMixin, dict):
     _format = 'dok'
 
     def __init__(self, arg1, shape=None, dtype=None, copy=False):
-        _spbase.__init__(self)
-        self._dict = {}
+        _spbase.__init__(self, arg1)
 
-        self.dtype = getdtype(dtype, default=float)
-        if isinstance(arg1, tuple) and isshape(arg1):  # (M,N)
-            M, N = arg1
-            self._shape = check_shape((M, N))
+        is_array = isinstance(self, sparray)
+        if isinstance(arg1, tuple) and isshape(arg1, allow_1d=is_array):
+            self._shape = check_shape(arg1, allow_1d=is_array)
+            self._dict = {}
+            self.dtype = getdtype(dtype, default=float)
         elif issparse(arg1):  # Sparse ctor
-            if arg1.format == self.format and copy:
-                arg1 = arg1.copy()
+            if arg1.format == self.format:
+                arg1 = arg1.copy() if copy else arg1
             else:
                 arg1 = arg1.todok()
 
             if dtype is not None:
                 arg1 = arg1.astype(dtype, copy=False)
 
-            self._dict.update(arg1)
-            self._shape = check_shape(arg1.shape)
+            self._dict = arg1._dict
+            self._shape = check_shape(arg1.shape, allow_1d=is_array)
             self.dtype = arg1.dtype
         else:  # Dense ctor
             try:
@@ -43,29 +44,29 @@ class _dok_base(_spbase, IndexMixin):
             except Exception as e:
                 raise TypeError('Invalid input format.') from e
 
-            if len(arg1.shape) != 2:
+            if arg1.ndim > 2:
                 raise TypeError('Expected rank <=2 dense array or matrix.')
 
-            d = self._coo_container(arg1, dtype=dtype).todok()
-            self._dict.update(d)
-            self._shape = check_shape(arg1.shape)
-            self.dtype = d.dtype
+            if arg1.ndim == 1:
+                if dtype is not None:
+                    arg1 = arg1.astype(dtype)
+                self._dict = {i: v for i, v in enumerate(arg1) if v != 0}
+                self.dtype = arg1.dtype
+            else:
+                d = self._coo_container(arg1, dtype=dtype).todok()
+                self._dict = d._dict
+                self.dtype = d.dtype
+            self._shape = check_shape(arg1.shape, allow_1d=is_array)
 
     def update(self, val):
         # Prevent direct usage of update
-        raise NotImplementedError("Direct modification to dok_array element "
-                                  "is not allowed.")
-
-    def _update(self, data):
-        """An update method for dict data defined for direct access to
-        `dok_array` data. Main purpose is to be used for efficient conversion
-        from other _spbase classes. Has no checking if `data` is valid."""
-        return self._dict.update(data)
+        raise NotImplementedError("Direct update to DOK sparse format is not allowed.")
 
     def _getnnz(self, axis=None):
         if axis is not None:
-            raise NotImplementedError("_getnnz over an axis is not implemented "
-                                      "for DOK format.")
+            raise NotImplementedError(
+                "_getnnz over an axis is not implemented for DOK format."
+            )
         return len(self._dict)
 
     def count_nonzero(self):
@@ -89,6 +90,24 @@ class _dok_base(_spbase, IndexMixin):
     def clear(self):
         return self._dict.clear()
 
+    def pop(self, /, *args):
+        return self._dict.pop(*args)
+
+    def __reversed__(self):
+        raise TypeError("reversed is not defined for dok_array type")
+
+    def __or__(self, other):
+        type_names = f"{type(self).__name__} and {type(other).__name__}"
+        raise TypeError(f"unsupported operand type for |: {type_names}")
+
+    def __ror__(self, other):
+        type_names = f"{type(self).__name__} and {type(other).__name__}"
+        raise TypeError(f"unsupported operand type for |: {type_names}")
+
+    def __ior__(self, other):
+        type_names = f"{type(self).__name__} and {type(other).__name__}"
+        raise TypeError(f"unsupported operand type for |: {type_names}")
+
     def popitem(self):
         return self._dict.popitem()
 
@@ -101,27 +120,56 @@ class _dok_base(_spbase, IndexMixin):
     def values(self):
         return self._dict.values()
 
-    def get(self, key, default=0.):
-        """This overrides the dict.get method, providing type checking
-        but otherwise equivalent functionality.
-        """
+    def get(self, key, default=0.0):
+        """This provides dict.get method functionality with type checking"""
+        if key in self._dict:
+            return self._dict[key]
+        if isintlike(key) and self.ndim == 1:
+            key = (key,)
+        if self.ndim != len(key):
+            raise IndexError(f'Index {key} length needs to match self.shape')
         try:
-            i, j = key
-            assert isintlike(i) and isintlike(j)
+            for i in key:
+                assert isintlike(i)
         except (AssertionError, TypeError, ValueError) as e:
-            raise IndexError('Index must be a pair of integers.') from e
-        if (i < 0 or i >= self.shape[0] or j < 0 or j >= self.shape[1]):
+            raise IndexError('Index must be or consist of integers.') from e
+        key = tuple(i + M if i < 0 else i for i, M in zip(key, self.shape))
+        if any(i < 0 or i >= M for i, M in zip(key, self.shape)):
             raise IndexError('Index out of bounds.')
+        if self.ndim == 1:
+            key = key[0]
         return self._dict.get(key, default)
 
+    # override IndexMixin.__getitem__ for 1d case until fully implemented
+    def __getitem__(self, key):
+        if self.ndim == 2:
+            return super().__getitem__(key)
+
+        if isinstance(key, tuple) and len(key) == 1:
+            key = key[0]
+        INT_TYPES = (int, np.integer)
+        if isinstance(key, INT_TYPES):
+            if key < 0:
+                key += self.shape[-1]
+            if key < 0 or key >= self.shape[-1]:
+                raise IndexError('index value out of bounds')
+            return self._get_int(key)
+        else:
+            raise IndexError('array/slice index for 1d dok_array not yet supported')
+
+    # 1D get methods
+    def _get_int(self, idx):
+        return self._dict.get(idx, self.dtype.type(0))
+
+    # 2D get methods
     def _get_intXint(self, row, col):
         return self._dict.get((row, col), self.dtype.type(0))
 
     def _get_intXslice(self, row, col):
-        return self._get_sliceXslice(slice(row, row+1), col)
+        return self._get_sliceXslice(slice(row, row + 1), col)
 
     def _get_sliceXint(self, row, col):
-        return self._get_sliceXslice(row, slice(col, col+1))
+        return self._get_sliceXslice(row, slice(col, col + 1))
 
     def _get_sliceXslice(self, row, col):
         row_start, row_stop, row_step = row.indices(self.shape[0])
@@ -184,6 +232,31 @@ class _dok_base(_spbase, IndexMixin):
                 newdok._dict[key] = v
         return newdok
 
+    # override IndexMixin.__setitem__ for 1d case until fully implemented
+    def __setitem__(self, key, value):
+        if self.ndim == 2:
+            return super().__setitem__(key, value)
+
+        if isinstance(key, tuple) and len(key) == 1:
+            key = key[0]
+        INT_TYPES = (int, np.integer)
+        if isinstance(key, INT_TYPES):
+            if key < 0:
+                key += self.shape[-1]
+            if key < 0 or key >= self.shape[-1]:
+                raise IndexError('index value out of bounds')
+            return self._set_int(key, value)
+        else:
+            raise IndexError('array index for 1d dok_array not yet provided')
+
+    # 1D set methods
+    def _set_int(self, idx, x):
+        if x:
+            self._dict[idx] = x
+        elif idx in self._dict:
+            del self._dict[idx]
+
+    # 2D set methods
     def _set_intXint(self, row, col, x):
         key = (row, col)
         if x:
@@ -207,27 +280,27 @@ class _dok_base(_spbase, IndexMixin):
         if isscalarlike(other):
             res_dtype = upcast_scalar(self.dtype, other)
             new = self._dok_container(self.shape, dtype=res_dtype)
-            # Add this scalar to every element.
-            M, N = self.shape
-            for key in itertools.product(range(M), range(N)):
+            # Add this scalar to each element.
+            for key in itertools.product(*[range(d) for d in self.shape]):
                 aij = self._dict.get(key, 0) + other
                 if aij:
                     new[key] = aij
-            # new.dtype.char = self.dtype.char
         elif issparse(other):
+            if other.shape != self.shape:
+                raise ValueError("Matrix dimensions are not equal.")
+            res_dtype = upcast(self.dtype, other.dtype)
+            new = self._dok_container(self.shape, dtype=res_dtype)
+            new._dict = self._dict.copy()
             if other.format == "dok":
-                if other.shape != self.shape:
-                    raise ValueError("Matrix dimensions are not equal.")
-                # We could alternatively set the dimensions to the largest of
-                # the two matrices to be summed.  Would this be a good idea?
-                res_dtype = upcast(self.dtype, other.dtype)
-                new = self._dok_container(self.shape, dtype=res_dtype)
-                new._dict.update(self._dict)
-                with np.errstate(over='ignore'):
-                    new._dict.update((k, new[k] + other[k]) for k in other.keys())
+                o_items = other.items()
             else:
-                csc = self.tocsc()
-                new = csc + other
+                other = other.tocoo()
+                if self.ndim == 1:
+                    o_items = zip(other.coords[0], other.data)
+                else:
+                    o_items = zip(zip(*other.coords), other.data)
+            with np.errstate(over='ignore'):
+                new._dict.update((k, new[k] + v) for k, v in o_items)
         elif isdense(other):
             new = self.todense() + other
         else:
@@ -235,35 +308,15 @@ class _dok_base(_spbase, IndexMixin):
         return new
 
     def __radd__(self, other):
-        if isscalarlike(other):
-            new = self._dok_container(self.shape, dtype=self.dtype)
-            M, N = self.shape
-            for key in itertools.product(range(M), range(N)):
-                aij = self._dict.get(key, 0) + other
-                if aij:
-                    new[key] = aij
-        elif issparse(other):
-            if other.format == "dok":
-                if other.shape != self.shape:
-                    raise ValueError("Matrix dimensions are not equal.")
-                new = self._dok_container(self.shape, dtype=self.dtype)
-                new._dict.update(self._dict)
-                new._dict.update((k, self[k] + other[k]) for k in other)
-            else:
-                csc = self.tocsc()
-                new = csc + other
-        elif isdense(other):
-            new = other + self.todense()
-        else:
-            return NotImplemented
-        return new
+        return self + other  # addition is comutative
 
     def __neg__(self):
         if self.dtype.kind == 'b':
-            raise NotImplementedError('Negating a sparse boolean matrix is not'
-                                      ' supported.')
+            raise NotImplementedError(
+                'Negating a sparse boolean matrix is not supported.'
+            )
         new = self._dok_container(self.shape, dtype=self.dtype)
-        new._dict.update((k, -self[k]) for k in self.keys())
+        new._dict.update((k, -v) for k, v in self.items())
         return new
 
     def _mul_scalar(self, other):
@@ -273,20 +326,41 @@ class _dok_base(_spbase, IndexMixin):
         new._dict.update(((k, v * other) for k, v in self.items()))
         return new
 
-    def _mul_vector(self, other):
-        # matrix * vector
-        result = np.zeros(self.shape[0], dtype=upcast(self.dtype, other.dtype))
+    def _matmul_vector(self, other):
+        res_dtype = upcast(self.dtype, other.dtype)
+
+        # vector @ vector
+        if self.ndim == 1:
+            if issparse(other):
+                if other.format == "dok":
+                    keys = self.keys() & other.keys()
+                else:
+                    keys = self.keys() & other.tocoo().coords[0]
+                return res_dtype(sum(self._dict[k] * other._dict[k] for k in keys))
+            elif isdense(other):
+                return res_dtype(sum(other[k] * v for k, v in self.items()))
+            else:
+                return NotImplemented
+
+        # matrix @ vector
+        result = np.zeros(self.shape[0], dtype=res_dtype)
         for (i, j), v in self.items():
             result[i] += v * other[j]
         return result
 
-    def _mul_multivector(self, other):
-        # matrix * multivector
-        result_shape = (self.shape[0], other.shape[1])
+    def _matmul_multivector(self, other):
         result_dtype = upcast(self.dtype, other.dtype)
-        result = np.zeros(result_shape, dtype=result_dtype)
+        # vector @ multivector
+        if self.ndim == 1:
+            # works for other 1d or 2d
+            return sum(v * other[j] for j, v in self._dict.items())
+
+        # matrix @ multivector
+        M = self.shape[0]
+        new_shape = (M,) if other.ndim == 1 else (M, other.shape[1])
+        result = np.zeros(new_shape, dtype=result_dtype)
         for (i, j), v in self.items():
-            result[i,:] += v * other[j,:]
+            result[i] += v * other[j]
         return result
 
     def __imul__(self, other):
@@ -315,26 +389,49 @@ class _dok_base(_spbase, IndexMixin):
         # is no shape attribute hence it is not possible to unpickle it.
         return dict.__reduce__(self)
 
+    def diagonal(self, k=0):
+        if self.ndim == 2:
+            return super().diagonal(k)
+        raise ValueError("diagonal requires two dimensions")
+
     def transpose(self, axes=None, copy=False):
+        if self.ndim == 1:
+            return self.copy()
+
         if axes is not None and axes != (1, 0):
-            raise ValueError("Sparse arrays/matrices do not support "
-                             "an 'axes' parameter because swapping "
-                             "dimensions is the only logical permutation.")
+            raise ValueError(
+                "Sparse arrays/matrices do not support "
+                "an 'axes' parameter because swapping "
+                "dimensions is the only logical permutation."
+            )
 
         M, N = self.shape
         new = self._dok_container((N, M), dtype=self.dtype, copy=copy)
-        new._dict.update((((right, left), val)
-                          for (left, right), val in self.items()))
+        new._dict.update((((right, left), val) for (left, right), val in self.items()))
         return new
 
     transpose.__doc__ = _spbase.transpose.__doc__
 
     def conjtransp(self):
-        """Return the conjugate transpose."""
+        """DEPRECATED: Return the conjugate transpose.
+
+        .. deprecated:: 1.14.0
+
+            `conjtransp` is deprecated and will be removed in v1.16.0.
+            Use `.T.conj()` instead.
+        """
+        msg = ("`conjtransp` is deprecated and will be removed in v1.16.0. "
+                   "Use `.T.conj()` instead.")
+        warn(msg, DeprecationWarning, stacklevel=2)
+
+        if self.ndim == 1:
+            new = self.tocoo()
+            new.data = new.data.conjugate()
+            return new
+
         M, N = self.shape
         new = self._dok_container((N, M), dtype=self.dtype)
-        new._dict.update((((right, left), np.conj(val))
-                          for (left, right), val in self.items()))
+        new._dict = {(right, left): np.conj(val) for (left, right), val in self.items()}
         return new
 
     def copy(self):
@@ -344,17 +441,28 @@ class _dok_base(_spbase, IndexMixin):
 
     copy.__doc__ = _spbase.copy.__doc__
 
+    @classmethod
+    def fromkeys(cls, iterable, value=1, /):
+        tmp = dict.fromkeys(iterable, value)
+        if isinstance(next(iter(tmp)), tuple):
+            shape = tuple(max(idx) + 1 for idx in zip(*tmp))
+        else:
+            shape = (max(tmp) + 1,)
+        result = cls(shape, dtype=type(value))
+        result._dict = tmp
+        return result
+
     def tocoo(self, copy=False):
-        if self.nnz == 0:
+        nnz = self.nnz
+        if nnz == 0:
             return self._coo_container(self.shape, dtype=self.dtype)
 
         idx_dtype = self._get_index_dtype(maxval=max(self.shape))
-        data = np.fromiter(self.values(), dtype=self.dtype, count=self.nnz)
-        row = np.fromiter((i for i, _ in self.keys()), dtype=idx_dtype, count=self.nnz)
-        col = np.fromiter((j for _, j in self.keys()), dtype=idx_dtype, count=self.nnz)
-        A = self._coo_container(
-            (data, (row, col)), shape=self.shape, dtype=self.dtype
-        )
+        data = np.fromiter(self.values(), dtype=self.dtype, count=nnz)
+        # handle 1d keys specially b/c not a tuple
+        inds = zip(*self.keys()) if self.ndim > 1 else (self.keys(),)
+        coords = tuple(np.fromiter(ix, dtype=idx_dtype, count=nnz) for ix in inds)
+        A = self._coo_container((data, coords), shape=self.shape, dtype=self.dtype)
         A.has_canonical_format = True
         return A
 
@@ -368,23 +476,50 @@ class _dok_base(_spbase, IndexMixin):
     todok.__doc__ = _spbase.todok.__doc__
 
     def tocsc(self, copy=False):
+        if self.ndim == 1:
+            raise NotImplementedError("tocsr() not valid for 1d sparse array")
         return self.tocoo(copy=False).tocsc(copy=copy)
 
     tocsc.__doc__ = _spbase.tocsc.__doc__
 
     def resize(self, *shape):
-        shape = check_shape(shape)
+        is_array = isinstance(self, sparray)
+        shape = check_shape(shape, allow_1d=is_array)
+        if len(shape) != len(self.shape):
+            # TODO implement resize across dimensions
+            raise NotImplementedError
+
+        if self.ndim == 1:
+            newN = shape[-1]
+            for i in list(self._dict):
+                if i >= newN:
+                    del self._dict[i]
+            self._shape = shape
+            return
+
         newM, newN = shape
         M, N = self.shape
         if newM < M or newN < N:
             # Remove all elements outside new dimensions
-            for (i, j) in list(self.keys()):
+            for i, j in list(self.keys()):
                 if i >= newM or j >= newN:
                     del self._dict[i, j]
         self._shape = shape
 
     resize.__doc__ = _spbase.resize.__doc__
 
+    # Added for 1d to avoid `tocsr` from _base.py
+    def astype(self, dtype, casting='unsafe', copy=True):
+        dtype = np.dtype(dtype)
+        if self.dtype != dtype:
+            result = self._dok_container(self.shape, dtype=dtype)
+            data = np.array(list(self._dict.values()), dtype=dtype)
+            result._dict = dict(zip(self._dict, data))
+            return result
+        elif copy:
+            return self.copy()
+        return self
+
 
 def isspmatrix_dok(x):
     """Is `x` of dok_array type?
@@ -466,7 +601,7 @@ class dok_array(_dok_base, sparray):
     """
 
 
-class dok_matrix(spmatrix, _dok_base, dict):
+class dok_matrix(spmatrix, _dok_base):
     """
     Dictionary Of Keys based sparse matrix.
 
@@ -517,6 +652,7 @@ class dok_matrix(spmatrix, _dok_base, dict):
     ...         S[i, j] = i + j    # Update element
 
     """
+
     def set_shape(self, shape):
         new_matrix = self.reshape(shape, copy=False).asformat(self.format)
         self.__dict__ = new_matrix.__dict__
@@ -526,3 +662,23 @@ class dok_matrix(spmatrix, _dok_base, dict):
         return self._shape
 
     shape = property(fget=get_shape, fset=set_shape)
+
+    def __reversed__(self):
+        return self._dict.__reversed__()
+
+    def __or__(self, other):
+        if isinstance(other, _dok_base):
+            return self._dict | other._dict
+        return self._dict | other
+
+    def __ror__(self, other):
+        if isinstance(other, _dok_base):
+            return self._dict | other._dict
+        return self._dict | other
+
+    def __ior__(self, other):
+        if isinstance(other, _dok_base):
+            self._dict |= other._dict
+        else:
+            self._dict |= other
+        return self
