@@ -474,7 +474,7 @@ class TestHyp2f1:
                     c=-15.5,
                     z=(1.1578947368421053-1.1578947368421053j),
                     expected=(0.9778506962676361+0.044083801141231616j),
-                    rtol=1e-12,
+                    rtol=3e-12,
                 ),
             ),
             pytest.param(
@@ -791,7 +791,7 @@ class TestHyp2f1:
                     c=2.0397202577726152,
                     z=(-0.3157894736842106+0.7368421052631575j),
                     expected=(54749.216391029935-23078.144720887536j),
-                    rtol=1e-15,
+                    rtol=2e-15,
                 ),
             ),
             pytest.param(
@@ -892,6 +892,340 @@ class TestHyp2f1:
         assert abs(z) < 1 and z.real < 0  # Tests the test
         assert_allclose(hyp2f1(a, b, c, z), expected, rtol=rtol)
 
+    @pytest.mark.parametrize(
+        "hyp2f1_test_case",
+        [
+            pytest.param(
+                Hyp2f1TestCase(
+                    a=16.25,
+                    b=-3.75,
+                    c=-3.5,
+                    z=(0.5263157894736841+0.7368421052631575j),
+                    expected=(-1279.4894322256655-2302.914821389276j),
+                    rtol=5e-14,
+                ),
+            ),
+            pytest.param(
+                Hyp2f1TestCase(
+                    a=-15.75,
+                    b=8.25,
+                    c=-1.5,
+                    z=(0.9473684210526314+0.3157894736842106j),
+                    expected=(-8889.452798586273-11961.162305065242j),
+                    rtol=5e-14,
+                ),
+            ),
+            pytest.param(
+                Hyp2f1TestCase(
+                    a=2.25,
+                    b=2.25,
+                    c=-1.5,
+                    z=(0.5263157894736841-0.736842105263158j),
+                    expected=(-236.58971357952055-238.5228224781136j),
+                    rtol=5e-15,
+                ),
+            ),
+            pytest.param(
+                Hyp2f1TestCase(
+                    a=-0.75,
+                    b=-7.75,
+                    c=-15.5,
+                    z=(0.5263157894736841+0.7368421052631575j),
+                    expected=(0.8116076584352279-0.29360565398246036j),
+                    rtol=5e-16,
+                ),
+            ),
+            pytest.param(
+                Hyp2f1TestCase(
+                    a=1.25,
+                    b=4.25,
+                    c=-0.5,
+                    z=(0.5263157894736841-0.736842105263158j),
+                    expected=(-28.119407485189985+98.89858821348005j),
+                    rtol=5e-15,
+                ),
+            ),
+            pytest.param(
+                Hyp2f1TestCase(
+                    a=-0.75,
+                    b=2.25,
+                    c=1.5,
+                    z=(0.5263157894736841+0.7368421052631575j),
+                    expected=(0.5311049067450484-0.9434347326448517j),
+                    rtol=5e-14,
+                ),
+            ),
+            pytest.param(
+                Hyp2f1TestCase(
+                    a=8.25,
+                    b=-15.75,
+                    c=-7.5,
+                    z=(0.9473684210526314+0.10526315789473673j),
+                    expected=(1262084.378141873+1775569.6338380123j),
+                    rtol=5e-14,
+                ),
+            ),
+            pytest.param(
+                Hyp2f1TestCase(
+                    a=-15.75,
+                    b=-7.75,
+                    c=-15.5,
+                    z=(0.5263157894736841-0.736842105263158j),
+                    expected=(-0.009810480794804165+0.3648997569257999j),
+                    rtol=1e-15,
+                ),
+            ),
+            pytest.param(
+                Hyp2f1TestCase(
+                    a=8.25,
+                    b=2.25,
+                    c=-3.5,
+                    z=(0.5263157894736841-0.736842105263158j),
+                    expected=(585660.8815535795-33646.68398590896j),
+                    rtol=5e-14,
+                ),
+            ),
+            pytest.param(
+                Hyp2f1TestCase(
+                    a=4.080187217753502,
+                    b=-3.9316537064827854,
+                    c=-15.963511401609862,
+                    z=(0.9473684210526314-0.10526315789473695j),
+                    expected=(181899621848365.2-173207123998705.7j),
+                    rtol=5e-14,
+                ),
+            ),
+            pytest.param(
+                Hyp2f1TestCase(
+                    a=-15.75,
+                    b=8.25,
+                    c=-0.5,
+                    z=(0.9473684210526314-0.10526315789473695j),
+                    expected=(0.04271686244952705-0.14087902824639406j),
+                    rtol=5e-14,
+                ),
+            ),
+            pytest.param(
+                Hyp2f1TestCase(
+                    a=-7.937789122896016,
+                    b=-1.92872979730171,
+                    c=-0.906685989801748,
+                    z=(0.9473684210526314-0.3157894736842106j),
+                    expected=(-449.5119088817207+320.1423128036188j),
+                    rtol=5e-14,
+                ),
+            ),
+            pytest.param(
+                Hyp2f1TestCase(
+                    a=-3.956227226099288,
+                    b=1.0561196186065624,
+                    c=8.031683612216888,
+                    z=(0.9473684210526314-0.10526315789473695j),
+                    expected=(0.6361479738012501+0.028575620091205088j),
+                    rtol=5e-15,
+                ),
+            ),
+            pytest.param(
+                Hyp2f1TestCase(
+                    a=4.25,
+                    b=16.25,
+                    c=16.5,
+                    z=(0.5263157894736841+0.7368421052631575j),
+                    expected=(-0.9038811840552261-1.5356250756164884j),
+                    rtol=1e-8,
+                ),
+                marks=pytest.mark.xfail(
+                    reason="Unhandled parameters."
+                )
+            ),
+            pytest.param(
+                Hyp2f1TestCase(
+                    a=8.25,
+                    b=-1.75,
+                    c=-1.5,
+                    z=(0.9473684210526314+0.3157894736842106j),
+                    expected=(653.0109150415394-4554.162605155542j),
+                    rtol=1e-15,
+                ),
+            ),
+            pytest.param(
+                Hyp2f1TestCase(
+                    a=-15.75,
+                    b=-3.75,
+                    c=4.5,
+                    z=(0.9473684210526314-0.10526315789473695j),
+                    expected=(118.7009859241035-34.18713648654642j),
+                    rtol=1e-15,
+                ),
+            ),
+            pytest.param(
+                Hyp2f1TestCase(
+                    a=4.25,
+                    b=-15.75,
+                    c=-3.5,
+                    z=(0.5263157894736841+0.7368421052631575j),
+                    expected=(-540204.4774526551+4970059.109251281j),
+                    rtol=1e-14,
+                ),
+            ),
+            pytest.param(
+                Hyp2f1TestCase(
+                    a=8.25,
+                    b=-15.75,
+                    c=-0.5,
+                    z=(0.5263157894736841-0.736842105263158j),
+                    expected=(2253490.972258385+3318620.683390017j),
+                    rtol=5e-13,
+                ),
+            ),
+            pytest.param(
+                Hyp2f1TestCase(
+                    a=16.25,
+                    b=-7.75,
+                    c=-7.5,
+                    z=(0.9473684210526314+0.3157894736842106j),
+                    expected=(-46159826.46716958-17880663.82218242j),
+                    rtol=5e-14,
+                ),
+            ),
+            pytest.param(
+                Hyp2f1TestCase(
+                    a=-0.9220024191881196,
+                    b=-7.949900487447654,
+                    c=-7.93846038215665,
+                    z=(0.9473684210526314-0.10526315789473695j),
+                    expected=(0.07116833581404514+0.11823358038036977j),
+                    rtol=5e-13,
+                ),
+            ),
+            pytest.param(
+                Hyp2f1TestCase(
+                    a=16.087593263474208,
+                    b=4.0013768449590685,
+                    c=-7.93846038215665,
+                    z=(0.7368421052631575+0.5263157894736841j),
+                    expected=(4.7724909620664006e+17-6.039064078946702e+16j),
+                    rtol=5e-13,
+                ),
+            ),
+            pytest.param(
+                Hyp2f1TestCase(
+                    a=1.25,
+                    b=-7.75,
+                    c=1.5,
+                    z=(0.9473684210526314-0.10526315789473695j),
+                    expected=(0.0188022179759303+0.002921737281641378j),
+                    rtol=5e-15,
+                ),
+            ),
+            pytest.param(
+                Hyp2f1TestCase(
+                    a=1.0272592605282642,
+                    b=1.0561196186065624,
+                    c=-7.93846038215665,
+                    z=(0.7368421052631575-0.5263157894736843j),
+                    expected=(-9203.462928334846+12390.110518017136j),
+                    rtol=5e-13,
+                ),
+            ),
+            pytest.param(
+                Hyp2f1TestCase(
+                    a=-3.75,
+                    b=-15.75,
+                    c=8.5,
+                    z=(0.7368421052631575+0.5263157894736841j),
+                    expected=(6.468457061368628+24.190040684917374j),
+                    rtol=5e-16,
+                ),
+            ),
+            pytest.param(
+                Hyp2f1TestCase(
+                    a=2.02764642551431,
+                    b=16.088264119063613,
+                    c=2.0397202577726152,
+                    z=(0.7368421052631575+0.5263157894736841j),
+                    expected=(2408.3451340186543-4275.257316636014j),
+                    rtol=5e-14,
+                ),
+            ),
+            pytest.param(
+                Hyp2f1TestCase(
+                    a=-3.75,
+                    b=-7.75,
+                    c=8.5,
+                    z=(0.7368421052631575-0.5263157894736843j),
+                    expected=(4.1379984626381345-5.183654781039423j),
+                    rtol=5e-15,
+                ),
+            ),
+            pytest.param(
+                Hyp2f1TestCase(
+                    a=8.25,
+                    b=-7.75,
+                    c=-0.5,
+                    z=(0.5263157894736841+0.7368421052631575j),
+                    expected=(-81177.775295738+56079.73286548954j),
+                    rtol=5e-14,
+                ),
+            ),
+            pytest.param(
+                Hyp2f1TestCase(
+                    a=4.080187217753502,
+                    b=2.050308316530781,
+                    c=-0.906685989801748,
+                    z=(0.9473684210526314+0.3157894736842106j),
+                    expected=(1192868.5068926765+3624210.8182139914j),
+                    rtol=5e-14,
+                ),
+            ),
+            pytest.param(
+                Hyp2f1TestCase(
+                    a=-7.937789122896016,
+                    b=-1.92872979730171,
+                    c=8.031683612216888,
+                    z=(0.5263157894736841+0.7368421052631575j),
+                    expected=(1.8286341846195202+1.9295255682312178j),
+                    rtol=5e-14,
+                ),
+            ),
+            pytest.param(
+                Hyp2f1TestCase(
+                    a=1.0272592605282642,
+                    b=1.0561196186065624,
+                    c=16.056809865262608,
+                    z=(0.7368421052631575-0.5263157894736843j),
+                    expected=(1.0514645669696452-0.0430834059440128j),
+                    rtol=5e-10,
+                ),
+            ),
+            pytest.param(
+                Hyp2f1TestCase(
+                    a=16.087593263474208,
+                    b=-15.964218273004214,
+                    c=2.0397202577726152,
+                    z=(0.5263157894736841+0.7368421052631575j),
+                    expected=(541983.236432269+288200.2043029435j),
+                    rtol=5e-13,
+                ),
+            ),
+            pytest.param(
+                Hyp2f1TestCase(
+                    a=2.25,
+                    b=8.25,
+                    c=1.5,
+                    z=(0.5263157894736841-0.736842105263158j),
+                    expected=(-10.931988086039945+1.9136272843579096j),
+                    rtol=1e-15,
+                ),
+            ),
+        ]
+    )
+    def test_region3(self, hyp2f1_test_case):
+        """0.9 <= |z| <= 1 and |1 - z| < 0.9."""
+        a, b, c, z, expected, rtol = hyp2f1_test_case
+        assert 0.9 <= abs(z) <= 1 and abs(1 - z) < 0.9  # Tests the test
+        assert_allclose(hyp2f1(a, b, c, z), expected, rtol=rtol)
+
     @pytest.mark.parametrize(
         "hyp2f1_test_case",
         [
@@ -1012,7 +1346,7 @@ class TestHyp2f1:
                     c=-7.93846038215665,
                     z=(0.4931034482758623+0.7965517241379312j),
                     expected=(-6318.553685853241-7133.416085202879j),
-                    rtol=1e-10,
+                    rtol=5e-9,
                 ),
             ),
             pytest.param(
@@ -1238,7 +1572,7 @@ class TestHyp2f1:
                     c=4.0013768449590685,
                     z=(0.3413793103448277-0.8724137931034484j),
                     expected=(528.5522951158454-1412.21630264791j),
-                    rtol=1e-15,
+                    rtol=5e-14,
                 ),
             ),
             pytest.param(
@@ -1358,7 +1692,7 @@ class TestHyp2f1:
                     c=-0.9629749245209605,
                     z=(0.3413793103448277+0.8724137931034486j),
                     expected=(-2764422.521269463-3965966.9965808876j),
-                    rtol=1e-15,
+                    rtol=5e-14,
                 ),
             ),
             pytest.param(
@@ -1378,7 +1712,7 @@ class TestHyp2f1:
                     c=-7.949900487447654,
                     z=(0.4172413793103451-0.8724137931034484j),
                     expected=(-2258.1590330318213+8860.193389158803j),
-                    rtol=1e-10,
+                    rtol=1.4e-10,
                 ),
             ),
         ]
@@ -1433,7 +1767,7 @@ class TestHyp2f1:
                     c=-15.964218273004214,
                     z=(0.6448275862068968+0.8724137931034486j),
                     expected=(85592537010.05054-8061416766688.324j),
-                    rtol=1e-14,
+                    rtol=2e-14,
                 ),
             ),
             pytest.param(
@@ -1556,7 +1890,7 @@ class TestHyp2f1:
                     c=-7.5,
                     z=(0.5689655172413794+0.8724137931034486j),
                     expected=(-0.3699450626264222+0.8732812475910993j),
-                    rtol=1e-15,
+                    rtol=5e-15,
                 ),
             ),
             pytest.param(
@@ -1779,7 +2113,7 @@ class TestHyp2f1:
                     c=16.5,
                     z=(0.18965517241379315+1.024137931034483j),
                     expected=(199.65868451496038+347.79384207302877j),
-                    rtol=1e-13,
+                    rtol=5e-12,
                 ),
             ),
             pytest.param(
@@ -1849,7 +2183,7 @@ class TestHyp2f1:
                     c=16.088264119063613,
                     z=(0.26551724137931054+1.024137931034483j),
                     expected=(-0.18515160890991517+0.7959014164484782j),
-                    rtol=1e-15,
+                    rtol=2e-15,
                 ),
             ),
             pytest.param(
@@ -2100,7 +2434,7 @@ class TestHyp2f1:
                     c=4.078873014294075,
                     z=(-0.9473684210526316-0.5263157894736843j),
                     expected=(-0.7505682955068583-1.1026583264249945j),
-                    rtol=1e-15,
+                    rtol=5e-14,
                 ),
             ),
             pytest.param(
@@ -2133,6 +2467,16 @@ class TestHyp2f1:
                     rtol=5e-14,
                 ),
             ),
+            pytest.param(
+                Hyp2f1TestCase(
+                    a=1,
+                    b=1,
+                    c=4,
+                    z=(3 + 4j),
+                    expected=(0.49234384000963544+0.6051340616612397j),
+                    rtol=5e-14,
+                ),
+            ),
         ]
     )
     def test_region6(self, hyp2f1_test_case):
