@@ -55,6 +55,8 @@ distcont = [
     ['invgamma', (4.0668996136993067,)],
     ['invgauss', (0.14546264555347513,)],
     ['invweibull', (10.58,)],
+    ['irwinhall', (10,)],
+    ['jf_skew_t', (8, 4)],
     ['johnsonsb', (4.3172675099141058, 3.1837781130785063)],
     ['johnsonsu', (2.554395574161155, 2.2482281679651965)],
     ['kappa4', (0.0, 0.0)],
@@ -87,13 +89,16 @@ distcont = [
     ['norminvgauss', (1.25, 0.5)],
     ['pareto', (2.621716532144454,)],
     ['pearson3', (0.1,)],
+    ['pearson3', (-2,)],
     ['powerlaw', (1.6591133289905851,)],
+    ['powerlaw', (0.6591133289905851,)],
     ['powerlognorm', (2.1413923530064087, 0.44639540782048337)],
     ['powernorm', (4.4453652254590779,)],
     ['rayleigh', ()],
     ['rdist', (1.6,)],
     ['recipinvgauss', (0.63004267809369119,)],
     ['reciprocal', (0.01, 1.25)],
+    ['rel_breitwigner', (36.545206797050334, )],
     ['rice', (0.7749725210111873,)],
     ['semicircular', ()],
     ['skewcauchy', (0.5,)],
@@ -105,6 +110,8 @@ distcont = [
     ['truncexpon', (4.6907725456810478,)],
     ['truncnorm', (-1.0978730080013919, 2.7306754109031979)],
     ['truncnorm', (0.1, 2.)],
+    ['truncpareto', (1.8, 5.3)],
+    ['truncpareto', (2, 5)],
     ['truncweibull_min', (2.5, 0.25, 1.75)],
     ['tukeylambda', (3.1321477856738267,)],
     ['uniform', ()],
@@ -119,6 +126,7 @@ distcont = [
 distdiscrete = [
     ['bernoulli',(0.3,)],
     ['betabinom', (5, 2.3, 0.63)],
+    ['betanbinom', (5, 9.3, 1)],
     ['binom', (5, 0.4)],
     ['boltzmann',(1.4, 19)],
     ['dlaplace', (0.8,)],  # 0.5
@@ -135,7 +143,7 @@ distdiscrete = [
     ['poisson', (0.6,)],
     ['randint', (7, 31)],
     ['skellam', (15, 8)],
-    ['zipf', (6.5,)],
+    ['zipf', (6.6,)],
     ['zipfian', (0.75, 15)],
     ['zipfian', (1.25, 10)],
     ['yulesimon', (11.0,)],
@@ -152,6 +160,7 @@ invdistdiscrete = [
     ['bernoulli', (1.5, )],
     ['binom', (10, 1.5)],
     ['betabinom', (10, -0.4, -0.5)],
+    ['betanbinom', (10, -0.4, -0.5)],
     ['boltzmann', (-1, 4)],
     ['dlaplace', (-0.5, )],
     ['geom', (1.5, )],
@@ -218,6 +227,10 @@ invdistcont = [
     ['invgamma', (-1, )],
     ['invgauss', (-1, )],
     ['invweibull', (-1, )],
+    ['irwinhall', (-1,)],
+    ['irwinhall', (0,)],
+    ['irwinhall', (2.5,)],
+    ['jf_skew_t', (-1, 0)],
     ['johnsonsb', (1, -2)],
     ['johnsonsu', (1, -2)],
     ['kappa4', (np.nan, 0)],
@@ -257,11 +270,14 @@ invdistcont = [
     ['semicircular', ()],
     ['skewnorm', (np.inf, )],
     ['studentized_range', (-1, 1)],
+    ['rel_breitwigner', (-2, )],
     ['t', (-1, )],
     ['trapezoid', (0, 2)],
     ['triang', (2, )],
     ['truncexpon', (-1, )],
     ['truncnorm', (10, 5)],
+    ['truncpareto', (-1, 5)],
+    ['truncpareto', (1.8, .5)],
     ['truncweibull_min', (-2.5, 0.25, 1.75)],
     ['tukeylambda', (np.nan, )],
     ['uniform', ()],
