@@ -17,6 +17,7 @@ from scipy.stats._hypotests import (epps_singleton_2samp, cramervonmises,
 from scipy.stats._mannwhitneyu import mannwhitneyu, _mwu_state
 from .common_tests import check_named_results
 from scipy._lib._testutils import _TestPythranFunc
+from scipy.stats._axis_nan_policy import SmallSampleWarning, too_small_1d_not_omit
 
 
 class TestEppsSingleton:
@@ -55,20 +56,17 @@ class TestEppsSingleton:
         assert_(p1 == p2 == p3)
 
     def test_epps_singleton_size(self):
-        # raise error if less than 5 elements
+        # warns if sample contains fewer than 5 elements
         x, y = (1, 2, 3, 4), np.arange(10)
-        assert_raises(ValueError, epps_singleton_2samp, x, y)
+        with pytest.warns(SmallSampleWarning, match=too_small_1d_not_omit):
+            res = epps_singleton_2samp(x, y)
+            assert_equal(res.statistic, np.nan)
+            assert_equal(res.pvalue, np.nan)
 
     def test_epps_singleton_nonfinite(self):
         # raise error if there are non-finite values
         x, y = (1, 2, 3, 4, 5, np.inf), np.arange(10)
         assert_raises(ValueError, epps_singleton_2samp, x, y)
-        x, y = np.arange(10), (1, 2, 3, 4, 5, np.nan)
-        assert_raises(ValueError, epps_singleton_2samp, x, y)
-
-    def test_epps_singleton_1d_input(self):
-        x = np.arange(100).reshape(-1, 1)
-        assert_raises(ValueError, epps_singleton_2samp, x, x)
 
     def test_names(self):
         x, y = np.arange(20), np.arange(30)
@@ -121,7 +119,7 @@ class TestCvm:
         # for large values of x and n, the series used to compute the cdf
         # converges slowly.
         # this leads to bug in R package goftest and MAPLE code that is
-        # the basis of the implemenation in scipy
+        # the basis of the implementation in scipy
         # note: cdf = 1 for x >= 1000/3 and n = 1000
         assert_(0.99999 < _cdf_cvm(333.3, 1000) < 1.0)
         assert_(0.99999 < _cdf_cvm(333.3) < 1.0)
@@ -134,11 +132,12 @@ class TestCvm:
         assert_(_cdf_cvm(res.statistic, n) > 1.0)
         assert_equal(res.pvalue, 0)
 
-    def test_invalid_input(self):
-        x = np.arange(10).reshape((2, 5))
-        assert_raises(ValueError, cramervonmises, x, "norm")
-        assert_raises(ValueError, cramervonmises, [1.5], "norm")
-        assert_raises(ValueError, cramervonmises, (), "norm")
+    @pytest.mark.parametrize('x', [(), [1.5]])
+    def test_invalid_input(self, x):
+        with pytest.warns(SmallSampleWarning, match=too_small_1d_not_omit):
+            res = cramervonmises(x, "norm")
+            assert_equal(res.statistic, np.nan)
+            assert_equal(res.pvalue, np.nan)
 
     def test_values_R(self):
         # compared against R package goftest, version 1.1.1
@@ -170,21 +169,27 @@ class TestCvm:
 
 
 class TestMannWhitneyU:
-    def setup_method(self):
-        _mwu_state._recursive = True
 
-    # All magic numbers are from R wilcox.test unless otherwise specied
+    # All magic numbers are from R wilcox.test unless otherwise specified
     # https://rdrr.io/r/stats/wilcox.test.html
 
     # --- Test Input Validation ---
 
+    @pytest.mark.parametrize('kwargs_update', [{'x': []}, {'y': []},
+                                               {'x': [], 'y': []}])
+    def test_empty(self, kwargs_update):
+        x = np.array([1, 2])  # generic, valid inputs
+        y = np.array([3, 4])
+        kwargs = dict(x=x, y=y)
+        kwargs.update(kwargs_update)
+        with pytest.warns(SmallSampleWarning, match=too_small_1d_not_omit):
+            res = mannwhitneyu(**kwargs)
+            assert_equal(res.statistic, np.nan)
+            assert_equal(res.pvalue, np.nan)
+
     def test_input_validation(self):
         x = np.array([1, 2])  # generic, valid inputs
         y = np.array([3, 4])
-        with assert_raises(ValueError, match="`x` and `y` must be of nonzero"):
-            mannwhitneyu([], y)
-        with assert_raises(ValueError, match="`x` and `y` must be of nonzero"):
-            mannwhitneyu(x, [])
         with assert_raises(ValueError, match="`use_continuity` must be one"):
             mannwhitneyu(x, y, use_continuity='ekki')
         with assert_raises(ValueError, match="`alternative` must be one of"):
@@ -367,21 +372,23 @@ class TestMannWhitneyU:
             for m, p in table.items():
                 # check p-value against table
                 u = np.arange(0, len(p))
-                assert_allclose(_mwu_state.cdf(k=u, m=m, n=n), p, atol=1e-3)
+                _mwu_state.set_shapes(m, n)
+                assert_allclose(_mwu_state.cdf(k=u), p, atol=1e-3)
 
                 # check identity CDF + SF - PMF = 1
                 # ( In this implementation, SF(U) includes PMF(U) )
                 u2 = np.arange(0, m*n+1)
-                assert_allclose(_mwu_state.cdf(k=u2, m=m, n=n)
-                                + _mwu_state.sf(k=u2, m=m, n=n)
-                                - _mwu_state.pmf(k=u2, m=m, n=n), 1)
+                assert_allclose(_mwu_state.cdf(k=u2)
+                                + _mwu_state.sf(k=u2)
+                                - _mwu_state.pmf(k=u2), 1)
 
                 # check symmetry about mean of U, i.e. pmf(U) = pmf(m*n-U)
-                pmf = _mwu_state.pmf(k=u2, m=m, n=n)
+                pmf = _mwu_state.pmf(k=u2)
                 assert_allclose(pmf, pmf[::-1])
 
                 # check symmetry w.r.t. interchange of m, n
-                pmf2 = _mwu_state.pmf(k=u2, m=n, n=m)
+                _mwu_state.set_shapes(n, m)
+                pmf2 = _mwu_state.pmf(k=u2)
                 assert_allclose(pmf, pmf2)
 
     def test_asymptotic_behavior(self):
@@ -580,11 +587,6 @@ class TestMannWhitneyU:
         assert_equal(res.statistic, statistic_exp)
         assert_allclose(res.pvalue, pvalue_exp)
 
-    def test_gh_6897(self):
-        # Test for correct behavior with empty input
-        with assert_raises(ValueError, match="`x` and `y` must be of nonzero"):
-            mannwhitneyu([], [])
-
     def test_gh_4067(self):
         # Test for correct behavior with all NaN input - default is propagate
         a = np.array([np.nan, np.nan, np.nan, np.nan, np.nan])
@@ -617,42 +619,43 @@ class TestMannWhitneyU:
                            method="asymptotic")
         assert_allclose(res, expected, rtol=1e-12)
 
-    def teardown_method(self):
-        _mwu_state._recursive = None
+    def test_gh19692_smaller_table(self):
+        # In gh-19692, we noted that the shape of the cache used in calculating
+        # p-values was dependent on the order of the inputs because the sample
+        # sizes n1 and n2 changed. This was indicative of unnecessary cache
+        # growth and redundant calculation. Check that this is resolved.
+        rng = np.random.default_rng(7600451795963068007)
+        m, n = 5, 11
+        x = rng.random(size=m)
+        y = rng.random(size=n)
+        _mwu_state.reset()  # reset cache
+        res = stats.mannwhitneyu(x, y, method='exact')
+        shape = _mwu_state.configurations.shape
+        assert shape[-1] == min(res.statistic, m*n - res.statistic) + 1
+        stats.mannwhitneyu(y, x, method='exact')
+        assert shape == _mwu_state.configurations.shape  # same when sizes are reversed
 
+        # Also, we weren't exploiting the symmmetry of the null distribution
+        # to its full potential. Ensure that the null distribution is not
+        # evaluated explicitly for `k > m*n/2`.
+        _mwu_state.reset()  # reset cache
+        stats.mannwhitneyu(x, 0*y, method='exact', alternative='greater')
+        shape = _mwu_state.configurations.shape
+        assert shape[-1] == 1  # k is smallest possible
+        stats.mannwhitneyu(0*x, y, method='exact', alternative='greater')
+        assert shape == _mwu_state.configurations.shape
 
-class TestMannWhitneyU_iterative(TestMannWhitneyU):
-    def setup_method(self):
-        _mwu_state._recursive = False
-
-    def teardown_method(self):
-        _mwu_state._recursive = None
-
-
-@pytest.mark.xslow
-def test_mann_whitney_u_switch():
-    # Check that mannwhiteneyu switches between recursive and iterative
-    # implementations at n = 500
-
-    # ensure that recursion is not enforced
-    _mwu_state._recursive = None
-    _mwu_state._fmnks = -np.ones((1, 1, 1))
-
-    rng = np.random.default_rng(9546146887652)
-    x = rng.random(5)
-
-    # use iterative algorithm because n > 500
-    y = rng.random(501)
-    stats.mannwhitneyu(x, y, method='exact')
-    # iterative algorithm doesn't modify _mwu_state._fmnks
-    assert np.all(_mwu_state._fmnks == -1)
-
-    # use recursive algorithm because n <= 500
-    y = rng.random(500)
-    stats.mannwhitneyu(x, y, method='exact')
-
-    # recursive algorithm has modified _mwu_state._fmnks
-    assert not np.all(_mwu_state._fmnks == -1)
+    @pytest.mark.parametrize('alternative', ['less', 'greater', 'two-sided'])
+    def test_permutation_method(self, alternative):
+        rng = np.random.default_rng(7600451795963068007)
+        x = rng.random(size=(2, 5))
+        y = rng.random(size=(2, 6))
+        res = stats.mannwhitneyu(x, y, method=stats.PermutationMethod(),
+                                 alternative=alternative, axis=1)
+        res2 = stats.mannwhitneyu(x, y, method='exact',
+                                  alternative=alternative, axis=1)
+        assert_allclose(res.statistic, res2.statistic, rtol=1e-15)
+        assert_allclose(res.pvalue, res2.pvalue, rtol=1e-15)
 
 
 class TestSomersD(_TestPythranFunc):
@@ -967,7 +970,7 @@ class TestSomersD(_TestPythranFunc):
         assert_equal(res.statistic, expected.statistic)
         assert_allclose(res.pvalue, expected.pvalue / 2)
 
-        with pytest.raises(ValueError, match="alternative must be 'less'..."):
+        with pytest.raises(ValueError, match="`alternative` must be..."):
             stats.somersd(x1, x2, alternative="ekki-ekki")
 
     @pytest.mark.parametrize("positive_correlation", (False, True))
@@ -1321,19 +1324,16 @@ class TestBoschlooExact:
 
 
 class TestCvm_2samp:
+    @pytest.mark.parametrize('args', [([], np.arange(5)),
+                                      (np.arange(5), [1])])
+    def test_too_small_input(self, args):
+        with pytest.warns(SmallSampleWarning, match=too_small_1d_not_omit):
+            res = cramervonmises_2samp(*args)
+            assert_equal(res.statistic, np.nan)
+            assert_equal(res.pvalue, np.nan)
+
     def test_invalid_input(self):
-        x = np.arange(10).reshape((2, 5))
         y = np.arange(5)
-        msg = 'The samples must be one-dimensional'
-        with pytest.raises(ValueError, match=msg):
-            cramervonmises_2samp(x, y)
-        with pytest.raises(ValueError, match=msg):
-            cramervonmises_2samp(y, x)
-        msg = 'x and y must contain at least two observations.'
-        with pytest.raises(ValueError, match=msg):
-            cramervonmises_2samp([], y)
-        with pytest.raises(ValueError, match=msg):
-            cramervonmises_2samp(y, [1])
         msg = 'method must be either auto, exact or asymptotic'
         with pytest.raises(ValueError, match=msg):
             cramervonmises_2samp(y, y, 'xyz')
@@ -1366,6 +1366,7 @@ class TestCvm_2samp:
         # The values are taken from Table 2, 3, 4 and 5
         assert_equal(_pval_cvm_2samp_exact(statistic, m, n), pval)
 
+    @pytest.mark.xslow
     def test_large_sample(self):
         # for large samples, the statistic U gets very large
         # do a sanity check that p-value is not 0, 1 or nan
@@ -1735,3 +1736,122 @@ class TestPoissonMeansTest:
         message = 'Alternative must be one of ...'
         with assert_raises(ValueError, match=message):
             stats.poisson_means_test(1, 2, 1, 2, alternative='error')
+
+
+class TestBWSTest:
+
+    def test_bws_input_validation(self):
+        rng = np.random.default_rng(4571775098104213308)
+
+        x, y = rng.random(size=(2, 7))
+
+        message = '`x` and `y` must be exactly one-dimensional.'
+        with pytest.raises(ValueError, match=message):
+            stats.bws_test([x, x], [y, y])
+
+        message = '`x` and `y` must not contain NaNs.'
+        with pytest.raises(ValueError, match=message):
+            stats.bws_test([np.nan], y)
+
+        message = '`x` and `y` must be of nonzero size.'
+        with pytest.raises(ValueError, match=message):
+            stats.bws_test(x, [])
+
+        message = 'alternative` must be one of...'
+        with pytest.raises(ValueError, match=message):
+            stats.bws_test(x, y, alternative='ekki-ekki')
+
+        message = 'method` must be an instance of...'
+        with pytest.raises(ValueError, match=message):
+            stats.bws_test(x, y, method=42)
+
+
+    def test_against_published_reference(self):
+        # Test against Example 2 in bws_test Reference [1], pg 9
+        # https://link.springer.com/content/pdf/10.1007/BF02762032.pdf
+        x = [1, 2, 3, 4, 6, 7, 8]
+        y = [5, 9, 10, 11, 12, 13, 14]
+        res = stats.bws_test(x, y, alternative='two-sided')
+        assert_allclose(res.statistic, 5.132, atol=1e-3)
+        assert_equal(res.pvalue, 10/3432)
+
+
+    @pytest.mark.parametrize(('alternative', 'statistic', 'pvalue'),
+                             [('two-sided', 1.7510204081633, 0.1264422777777),
+                              ('less', -1.7510204081633, 0.05754662004662),
+                              ('greater', -1.7510204081633, 0.9424533799534)])
+    def test_against_R(self, alternative, statistic, pvalue):
+        # Test against R library BWStest function bws_test
+        # library(BWStest)
+        # options(digits=16)
+        # x = c(...)
+        # y = c(...)
+        # bws_test(x, y, alternative='two.sided')
+        rng = np.random.default_rng(4571775098104213308)
+        x, y = rng.random(size=(2, 7))
+        res = stats.bws_test(x, y, alternative=alternative)
+        assert_allclose(res.statistic, statistic, rtol=1e-13)
+        assert_allclose(res.pvalue, pvalue, atol=1e-2, rtol=1e-1)
+
+    @pytest.mark.parametrize(('alternative', 'statistic', 'pvalue'),
+                             [('two-sided', 1.142629265891, 0.2903950180801),
+                              ('less', 0.99629665877411, 0.8545660222131),
+                              ('greater', 0.99629665877411, 0.1454339777869)])
+    def test_against_R_imbalanced(self, alternative, statistic, pvalue):
+        # Test against R library BWStest function bws_test
+        # library(BWStest)
+        # options(digits=16)
+        # x = c(...)
+        # y = c(...)
+        # bws_test(x, y, alternative='two.sided')
+        rng = np.random.default_rng(5429015622386364034)
+        x = rng.random(size=9)
+        y = rng.random(size=8)
+        res = stats.bws_test(x, y, alternative=alternative)
+        assert_allclose(res.statistic, statistic, rtol=1e-13)
+        assert_allclose(res.pvalue, pvalue, atol=1e-2, rtol=1e-1)
+
+    def test_method(self):
+        # Test that `method` parameter has the desired effect
+        rng = np.random.default_rng(1520514347193347862)
+        x, y = rng.random(size=(2, 10))
+
+        rng = np.random.default_rng(1520514347193347862)
+        method = stats.PermutationMethod(n_resamples=10, random_state=rng)
+        res1 = stats.bws_test(x, y, method=method)
+
+        assert len(res1.null_distribution) == 10
+
+        rng = np.random.default_rng(1520514347193347862)
+        method = stats.PermutationMethod(n_resamples=10, random_state=rng)
+        res2 = stats.bws_test(x, y, method=method)
+
+        assert_allclose(res1.null_distribution, res2.null_distribution)
+
+        rng = np.random.default_rng(5205143471933478621)
+        method = stats.PermutationMethod(n_resamples=10, random_state=rng)
+        res3 = stats.bws_test(x, y, method=method)
+
+        assert not np.allclose(res3.null_distribution, res1.null_distribution)
+
+    def test_directions(self):
+        # Sanity check of the sign of the one-sided statistic
+        rng = np.random.default_rng(1520514347193347862)
+        x = rng.random(size=5)
+        y = x - 1
+
+        res = stats.bws_test(x, y, alternative='greater')
+        assert res.statistic > 0
+        assert_equal(res.pvalue, 1 / len(res.null_distribution))
+
+        res = stats.bws_test(x, y, alternative='less')
+        assert res.statistic > 0
+        assert_equal(res.pvalue, 1)
+
+        res = stats.bws_test(y, x, alternative='less')
+        assert res.statistic < 0
+        assert_equal(res.pvalue, 1 / len(res.null_distribution))
+
+        res = stats.bws_test(y, x, alternative='greater')
+        assert res.statistic < 0
+        assert_equal(res.pvalue, 1)
