@@ -1,14 +1,14 @@
+#!/usr/bin/env python3
 """
 Process f2py template files (`filename.pyf.src` -> `filename.pyf`)
 
 Usage: python generate_pyf.py filename.pyf.src -o filename.pyf
 """
 
+import argparse
 import os
-import sys
 import re
 import subprocess
-import argparse
 
 
 # START OF CODE VENDORED FROM `numpy.distutils.from_template`
@@ -283,7 +283,7 @@ def main():
 
     # Now invoke f2py to generate the C API module file
     if args.infile.endswith(('.pyf.src', '.pyf')):
-        p = subprocess.Popen([sys.executable, '-m', 'numpy.f2py', fname_pyf,
+        p = subprocess.Popen(['f2py', fname_pyf,
                             '--build-dir', outdir_abs], #'--quiet'],
                             stdout=subprocess.PIPE, stderr=subprocess.PIPE,
                             cwd=os.getcwd())
